let server,
    resolve,
    readyPromise = new Promise(done => {
        resolve = done;
    });

module.exports = {
    ready: readyPromise,
    close: () => {
        if (server) {
            server.close();
        }
    },
    start: function (config, webpackConfig) {
        if (!process.env.NODE_ENV) {
            process.env.NODE_ENV = JSON.parse(config.dev.env.NODE_ENV);
        }
        let webpack = require("webpack"),
            devserver = require("gs-devserver"),
            app;

        app = devserver.createExpress(config, function (app) {
            let compiler,
                devMiddleware,
                hotMiddleware;

            compiler = webpack(webpackConfig);

            devMiddleware = require("webpack-dev-middleware")(compiler, {
                publicPath: webpackConfig.output.publicPath,
                quiet: true
            });

            hotMiddleware = require("webpack-hot-middleware")(compiler, {
                log: false,
                heartbeat: 2000
            });

            // force page reload when html-webpack-plugin template changes
            compiler.plugin("compilation", function (compilation) {
                compilation.plugin("html-webpack-plugin-after-emit", function (data, cb) {
                    hotMiddleware.publish({action: "reload"});
                    cb();
                });
            });

            // handle fallback for HTML5 history API
            const historyApiOptions = config.dev.historyApiOptions;
            app.use(require("connect-history-api-fallback")(historyApiOptions));

            // serve webpack bundle output
            app.use(devMiddleware);

            // enable hot-reload and state-preserving
            // compilation error display
            app.use(hotMiddleware);

            const opener = devserver.opener;
            delete devserver.opener;
            devMiddleware.waitUntilValid(() => {
                if (server) {
                    // when env is testing, do not need open it
                    if (process.env.NODE_ENV !== "testing") {
                        opener(config);
                    }
                    // webpack dev server has ready
                    resolve();
                }
            });
        });

        server = devserver.createHttpsServer(config, app);

        return server;
    }
};
