const
    merge = require("webpack-merge"),
    HtmlWebpackPlugin = require("html-webpack-plugin"),
    OptimizeCSSPlugin = require("optimize-css-assets-webpack-plugin"),
    ExtractTextPlugin = require("extract-text-webpack-plugin");

function isAry (obj) {
    return Object.prototype.toString.call(obj) === "[object Array]";
}

module.exports = function (config, options) {
    const isProd = process.env.NODE_ENV === "production",
        babelLoaderPlugins = [
            "transform-runtime"
        ],
        cfgBabelLoaderPlugins = isProd ? config.dev.babelLoaderPlugins : config.build.babelLoaderPlugins;

    if (isAry(cfgBabelLoaderPlugins)) {
        cfgBabelLoaderPlugins.forEach(plugin => {
            babelLoaderPlugins.push(plugin);
        });
    }

    let baseCfg = {
        entry: isProd ? config.build.entry : config.dev.entry,
        output: merge({
            path: isProd ? config.build.assetsRoot : config.dev.assetsRoot,
            publicPath: isProd ? config.build.assetsPublicPath : config.dev.assetsPublicPath
        }, options.output),
        // https://webpack.js.org/configuration/resolve/#resolvesymlinks
        resolve: {symlinks: false},
        module: {
            loaders: [{
                test: /\.js$/,
                exclude: /node_modules/,
                loader: "babel-loader",
                query: {
                    presets: ["env", "react"],
                    plugins: babelLoaderPlugins
                }
            }, {
                // handle translation properties file
                test: /\.properties$/,
                use: [
                    {
                        loader: "properties-loader"
                    }
                ]
            }, {
                test: /(\.css)$/,
                use: ExtractTextPlugin.extract({
                    fallback: "style-loader",
                    use: [{
                        loader: "css-loader",
                        options: merge({}, options.cssLoader)
                    }]})
            }, {
                test: /(\.less)$/,
                use: ExtractTextPlugin.extract({
                    fallback: [{
                        loader: "style-loader"
                    }],
                    use: [
                        "css-loader",
                        "less-loader"
                    ]
                })
            }, {
                test: /\.(png|jpe?g|gif|svg)(\?.*)?$/,
                loader: "url-loader",
                options: merge({}, options.urlLoader)
            }]
        },
        plugins: [
            new HtmlWebpackPlugin(merge({
                // raw loader for do not translate default template syntax
                template: isProd ? config.build.template : config.dev.template,
                filename: isProd ? config.build.index : config.dev.index,
                inject: "body"
            }, options.htmlWebpackPlugin, isProd ? config.build.htmlwebpackplugin : config.dev.htmlwebpackplugin)),
            new ExtractTextPlugin(merge({
                disable: false,
                allChunks: true
            }, options.extractTextPlugin)),
            // https://www.npmjs.com/package/optimize-css-assets-webpack-plugin
            new OptimizeCSSPlugin({
                cssProcessorOptions: {
                    safe: true
                }
            })
        ]
    };

    if (isProd ? config.build.bundleAnalyzerReport : config.dev.bundleAnalyzerReport) {
        const BundleAnalyzerPlugin = require("webpack-bundle-analyzer").BundleAnalyzerPlugin;
        baseCfg.plugins.push(new BundleAnalyzerPlugin());
    }

    // customize plugin
    let plugins = isProd ? config.build.plugins : config.dev.plugins;
    if (plugins) {
        if (!isAry(plugins)) {
            plugins = [plugins];
        }

        plugins.forEach(plugin => {
            baseCfg.plugins.push(plugin);
        });
    }

    return baseCfg;
};