const options = {
    output: {
        filename: "js/[name].js"
    },
    cssLoader: {},
    urlLoader: {},
    htmlWebpackPlugin: {
        minify: false
    },
    extractTextPlugin: {
        filename: "css/main.css"
    }
};

module.exports = function (config) {
    if (!process.env.NODE_ENV) {
        process.env.NODE_ENV = JSON.parse(config.dev.env.NODE_ENV);
    }

    let baseCfg = require("./webpack.base.conf"),
        webpackCfg = baseCfg(config, options);

    const
        webpack = require("webpack"),
        FriendlyErrorsPlugin = require("friendly-errors-webpack-plugin");

    // http://webpack.github.io/docs/hot-module-replacement-with-webpack.html
    webpackCfg.entry = ["webpack-hot-middleware/client?reload=true"].concat(webpackCfg.entry);

    // cheap-module-eval-source-map is faster for development
    webpackCfg.devtool = "#cheap-module-eval-source-map";

    webpackCfg.plugins = webpackCfg.plugins.concat([
        new webpack.DefinePlugin({
            __DEBUG__: true,
            "process.env": config.dev.env
        }),
        // https://github.com/glenjamin/webpack-hot-middleware#installation--usage
        new webpack.HotModuleReplacementPlugin(),
        new webpack.NoEmitOnErrorsPlugin(),
        new FriendlyErrorsPlugin()
    ]);

    return webpackCfg;
};