const UglifyJsPlugin = require("uglifyjs-webpack-plugin");
module.exports = function (config) {
    if (!process.env.NODE_ENV) {
        process.env.NODE_ENV = JSON.parse(config.build.env.NODE_ENV);
    }

    const options = {
        output: {
            filename: "js/[name]_[hash].js"
        },
        cssLoader: {
            minimize: true,
            url: true
        },
        urlLoader: {
            limit: 10000
        },
        htmlWebpackPlugin: {
            minify: {
                removeComments: true,
                collapseWhitespace: true,
                minifyJS: true
            }
        },
        extractTextPlugin: {
            filename: "css/main_[hash].css"
        }
    };

    let baseCfg = require("./webpack.base.conf"),
        webpackCfg = baseCfg(config, options);

    const
        path = require("path"),
        webpack = require("webpack");

    webpackCfg.plugins = webpackCfg.plugins.concat([
        new webpack.DefinePlugin({
            "process.env": config.build.env
        }),
        new UglifyJsPlugin(),

        // babel plugin transform runtime will auto polyfill global API, like Promise
        // so we no need provide extra promise implementation suck as Bluebird

        // keep module.id stable when vender modules does not change
        new webpack.HashedModuleIdsPlugin(),
        // split vendor js into its own file
        new webpack.optimize.CommonsChunkPlugin({
            name: "vendor",
            minChunks: function (module/** , count */) {
                // any required modules inside node_modules are extracted to vendor
                return (
                    module.resource &&
                    /\.js$/.test(module.resource) &&
                    module.resource.indexOf(
                        path.join(__dirname, "../../../../node_modules")
                    ) === 0
                );
            }
        }),
        // extract webpack runtime and module manifest to its own file in order to
        // prevent vendor hash from being updated whenever app bundle is updated
        new webpack.optimize.CommonsChunkPlugin({
            name: "manifest",
            chunks: ["vendor"]
        })
    ]);

    return webpackCfg;
};