const path = require("path"),
    utils = require("./utils");
let baseCfg;

function resolve (dir) {
    // root package.json directory
    return path.join(__dirname, "../../../..", dir);
}

module.exports = function (config) {
    if (!baseCfg) {
        utils.config(config);

        let vueLoaderConfig = require("./vue-loader.conf")(config),
            isProd = process.env.NODE_ENV === "production";

        baseCfg = {
            entry: isProd ? config.build.entry : config.dev.entry,
            output: {
                path: isProd ? config.build.assetsRoot : config.dev.assetsRoot,
                filename: "[name].js",
                publicPath: isProd ? config.build.assetsPublicPath : config.dev.assetsPublicPath
            },
            resolve: {
                extensions: [".js", ".vue", ".json"],
                alias: {
                    "vue$": "vue/dist/vue.esm.js",
                    "@": resolve("src")
                },
                // https://webpack.js.org/configuration/resolve/#resolvesymlinks
                symlinks: false
            },
            module: {
                rules: [{
                    test: /\.vue$/,
                    loader: "vue-loader",
                    options: vueLoaderConfig
                }, {
                    test: /\.js$/,
                    loader: "babel-loader",
                    include: [resolve("src"), resolve("test")]
                }, {
                    test: /\.(png|jpe?g|gif|svg)(\?.*)?$/,
                    loader: "url-loader",
                    options: {
                        limit: 10000,
                        name: utils.assetsPath("img/[name].[hash:7].[ext]")
                    }
                }, {
                    test: /\.(mp4|webm|ogg|mp3|wav|flac|aac)(\?.*)?$/,
                    loader: "url-loader",
                    options: {
                        limit: 10000,
                        name: utils.assetsPath("media/[name].[hash:7].[ext]")
                    }
                }, {
                    test: /\.(woff2?|eot|ttf|otf)(\?.*)?$/,
                    loader: "url-loader",
                    options: {
                        limit: 10000,
                        name: utils.assetsPath("fonts/[name].[hash:7].[ext]")
                    }
                }]
            }
        };
    }
    return baseCfg;
};