const
    utils = require("./utils"),
    webpack = require("webpack"),
    merge = require("webpack-merge"),
    HtmlWebpackPlugin = require("html-webpack-plugin"),
    HtmlWebpackInlineSourcePlugin = require("html-webpack-inline-source-plugin"),
    FriendlyErrorsPlugin = require("friendly-errors-webpack-plugin");

let devCfg;

module.exports = function (config) {
    if (!devCfg) {
        let baseWebpackConfig = require("./webpack.base.conf")(config);
        // add hot-reload related code to entry chunks
        Object.keys(baseWebpackConfig.entry).forEach(function (name) {
            baseWebpackConfig.entry[name] = ["./node_modules/gs-webpack/build/dev-client"].concat(baseWebpackConfig.entry[name]);
        });

        // https://github.com/ampedandwired/html-webpack-plugin
        let htmlWebpackPlugins = [],
            configPlugins = [];
        if (Array.isArray(config.dev.htmlwebpackplugin)) {
            configPlugins = config.dev.htmlwebpackplugin;
        } else {
            configPlugins = [config.dev.htmlwebpackplugin || {}];
        }
        configPlugins.forEach(plugin => {
            htmlWebpackPlugins.push(
                new HtmlWebpackPlugin(merge({
                    filename: config.dev.index,
                    // raw loader for do not translate default template syntax
                    template: config.dev.template,
                    inject: true,
                    // embed css inline
                    inlineSource: ".(css)$"
                }, plugin)));
        });

        devCfg = merge(baseWebpackConfig, {
            module: {
                rules: utils.styleLoaders({sourceMap: config.dev.cssSourceMap})
            },
            // cheap-module-eval-source-map is faster for development
            devtool: "#cheap-module-eval-source-map",
            plugins: [
                new webpack.DefinePlugin({
                    "process.env": config.dev.env
                }),
                // https://github.com/glenjamin/webpack-hot-middleware#installation--usage
                new webpack.HotModuleReplacementPlugin(),
                new webpack.NoEmitOnErrorsPlugin(),
                new HtmlWebpackInlineSourcePlugin(),
                new FriendlyErrorsPlugin()
            ].concat(htmlWebpackPlugins)
        });
    }
    return devCfg;
};