const
    path = require("path"),
    fs = require("fs"),
    readline = require("readline"),
    fse = require("fs-extra"),
    _ = require("lodash"),
    fn = require("gs-common/be/functions"),
    logging = require("gs-common/fe/Logging"),
    OPTS = require("./opts");

function writeTemplate (ctx, target) {
    const tmplDir = path.join(__dirname, "template"),
        tempDir = path.join(__dirname, "temp"),
        tmplMapping = {
            pkg: ["package.json", "README.md", "build/localizer.js"],
            cfg: "config/index.js",
            pack: "build/pack.js",
            gsModules: ["build/clean.js", "build/copyLoading.js"]

        };

    fn.rmdir(tempDir);
    fse.copySync(tmplDir, tempDir);

    Object.entries(tmplMapping)
        .forEach(([key, tmplPaths]) => {
            if (typeof tmplPaths === "string") {
                tmplPaths = [tmplPaths];
            }
            tmplPaths.forEach((tmplPath, i) => {
                tmplPath = path.join(tempDir, tmplPath);
                // remove localizer build file
                if (key === "pkg" && i === 2 && !ctx.pkg.localizer) {
                    fs.unlinkSync(tmplPath);
                    return;
                }

                logging.info("read template", tmplPath);

                let tmplText = fs.readFileSync(tmplPath, "utf-8");
                const compiledTmpl = _.template(tmplText, {
                    interpolate: /\{\{(.+?)\}\}/g
                });

                let result = compiledTmpl(ctx[key]);

                if (key === "pkg" && i === 0) {
                    if (!ctx.pkg.localizer) {
                        result = result.replace(`,
        "localizer": "node ./build/localizer.js",
        "localizer:validate": "node ./build/localizer.js --validate"`, "")
                            .replace(`,
        "gs-localizer": "0.2.0"`, "")
                            .replace(`,
        "properties-loader": "0.0.1"`, "")
                            .replace(/ localizer /g, " ");
                    }
                }

                logging.info("write template", tmplPath);
                fs.writeFileSync(tmplPath, result);
            });
        });

    [
        "assets",
        "components",
        "model",
        "service",
        "styles"
    ].forEach(dir => {
        dir = path.join(tempDir, "src", dir);
        logging.info("make dir", dir);
        fs.mkdirSync(dir);
    });

    logging.info("copy template files to", target);
    fse.copySync(tempDir, target);
    logging.info("clean up temp dir", tempDir);
    fn.rmdir(tempDir);
}

function readOpts (opts) {
    let
        resolve,
        promise = new Promise((res => {
            resolve = res;
        })),
        rl = readline.createInterface(process.stdin, process.stdout),
        keys = Object.keys(opts).filter(key => typeof key === "string"),

        question = function () {
            const key = keys.shift();

            rl.question(opts.question(key), function (input) {
                input = input || "";
                input = input.trim();
                // without input
                if (!input) {
                    if (opts.stop(key, input)) {
                        rl.close();
                        resolve();
                    } else {
                        question();
                    }
                    return;
                }

                // convert string to other data type
                if (opts.convert) {
                    input = opts.convert(key, input);
                }
                opts[key] = input;

                if (opts.stop(key, input)) {
                    rl.close();
                    resolve();
                } else {
                    question();
                }
            });
        };

    question();

    return promise;
}

function run () {
    let target = process.argv[2];
    if (!target) {
        logging.info("no target folder given");
        logging.info("e.g. node vue-cli.js test");
        return;
    }
    logging.info("target folder", target);
    // uniform path to absolute
    if (!path.isAbsolute(target)) {
        target = path.join(process.cwd(), target);
        logging.info("uniform absolute", target);
    }
    if (fs.existsSync(target)) {
        logging.info(`target folder "${target}" has existed`);
        return;
    }

    (async function () {
        logging.debug("opts before", OPTS);
        const keys = Object.keys(OPTS);
        for (let i = 0; i < keys.length; i++) {
            const key = keys[i];
            if (!OPTS[key].question) {
                // set pack option from cfg
                if (OPTS.cfg.basePath) {
                    const basePath = OPTS.cfg.basePath;
                    OPTS.pack.basePath = basePath;
                    OPTS.pack.cwd = basePath.substring(0, basePath.lastIndexOf("/"));
                    OPTS.pack.dir = basePath.substring(basePath.lastIndexOf("/") + 1);
                }
                continue;
            }
            await readOpts(OPTS[key]);
        }
        logging.debug("opts after", OPTS);
        writeTemplate(OPTS, target);
    }());
}

module.exports = {
    run: run
};

if (typeof require !== "undefined" && require.main === module) {
    run();
}