const util = require("gs-common/be/functions");
// override default hostname (localhost)
process.env.HOSTNAME = process.env.HOSTNAME || util.hostname();
process.env.PROXY_HOST = process.env.PROXY_HOST || "https://garminsports-web-inte.cn1.cf.garmin.com";

let path = require("path"),
    basePath = "{{basePath}}",
    config = {
        build: {
            // redefine to static dir
            index: path.resolve(__dirname, `../dist${basePath}/index.html`),
            assetsRoot: path.resolve(__dirname, `../dist${basePath}`),
            assetsPublicPath: `${basePath}/`,
            htmlwebpackplugin: [
                {
                    minify: {
                        minifyJS: true
                    }
                }
                // More than one template config demo
                // {
                //     filename: "index-zh_CN.html",
                //     template: "!!raw-loader!index-zh_CN.html",
                //     minify: {
                //         minifyJS: true
                //     }
                // }
            ],
            bundleAnalyzerReport: false
        },
        dev: {
            assetsRoot: path.resolve(__dirname, `../dist${basePath}`),
            assetsPublicPath: `${basePath}/`,
            templatePath: `${basePath}/index.html`,
            open: `${basePath}/index.html`,
            auth: false,
            htmlwebpackplugin: [
                {
                    filename: "index.html",
                    template: "!!raw-loader!index.html",
                }
            ]
        }
    };

config = require("gs-webpack/vue/config")(config);

// override gs-webpack/vue/config proxy config
const contentProxy = config.dev.proxyTable["/content/**"];
if (contentProxy) {
    config.dev.proxyTable["/content/**"] = {
        // exclude startsWith basePath path
        filter: function (pathname) {
            return /^\/content\//.test(pathname) &&
                !new RegExp(`^${basePath}/`).test(pathname);
        },
        target: contentProxy.target,
        secure: false
    };
}

/**
 * proxyTable["/signin"] original config is in gs-deveserver/config/proxycfg.js
 * modify this config to resolve login failed issue on integration and test server
 */
const signinProxy = config.dev.proxyTable["/signin"];
if (signinProxy) {
    config.dev.proxyTable["/signin"] = {
        // exclude startsWith basePath path
        filter: function (pathname) {
            return /\/signin/.test(pathname);
        },
        target: signinProxy,
        secure: false,
        xfwd: true,
        changeOrigin: true
    };
}

//////////////////////////////////////////////////////
// For server deploy to PCF
const keys = Object.keys(config.dev.proxyTable);
keys.forEach(key => {
    let configItem = config.dev.proxyTable[key];
    if (typeof configItem === 'string') {
        configItem = {
            target: configItem
        };
    }
    configItem = Object.assign(configItem, {
        secure: false,
        changeOrigin: true,
        xfwd: true
        // hostRewrite: process.env.HOSTNAME + ":" + process.env.PORT,
        // cookieDomainRewrite: process.env.HOSTNAME + process.env.PORT,
        // protocolRewrite: "https:"
    });
    config.dev.proxyTable[key] = configItem;
});
//////////////////////////////////////////////////////

module.exports = config;